﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Trolleybus.Generics
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetGeneric<T>
        where T : class
    {
        /// <summary>
        /// Список объектов, которые храним
        /// </summary>
        private readonly List<T>? _places;
        /// <summary>
        /// Количество объектов в списке
        /// </summary>
        public int Count => _places.Count;
        /// <summary>
        /// Максимальное количество объектов в списке
        /// </summary>
        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(count);
        }
        /// <summary>
        /// Добавление объекта в начало набора
        /// </summary>
        /// <param name="bus">Добавляемый автобус</param>
        /// <returns></returns>
        public int Insert(T bus)
        {
            if (Count + 1 <= _maxCount)
            {
                _places.Insert(0, bus);
                //0 в данном случае место, на котоорое добавился элемннт, вставка прошла успешно
                return 0;
            }
            //если при добавлении в списке станет больше макс. кол-ва элементов, то как бы вставлять будет некуда
            return -1;
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="bus">Добавляемый автобус</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T bus, int position)
        {
            if (position >= _maxCount || position < 0)
            {
                //позиция неверная, значит вставить нельзя
                return -1;
            }
            if (Count + 1 <= _maxCount)
            {
                _places.Insert(position, bus);
                //место в списке, по которому вставили, вставка прошла успешно
                return position;
            }
            //места в списке нет
            return -1;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public bool Remove(int position)
        {
            if (position >= Count || position < 0)
            {
                return false;
            }
            _places.RemoveAt(position);
            return true;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T? this[int position]
        {
            get
            {
                if (position >= Count || position < 0)
                {
                    return null;
                }
                return _places[position];
            }
            set
            {
                if (position >= Count || position < 0)
                {
                    return;
                }
                _places[position] = value;
            }
        }
        /// <summary>
        /// Проход по списку
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T?> GetBuses(int? maxBuses = null)
        {
            for (int i = 0; i < Count; ++i)
            {
                yield return _places[i];
                if (maxBuses.HasValue && i == maxBuses.Value)
                {
                    yield break;
                }
            }
        }
    }
}
